% In this script we show an example of usage of the core part of the
% deblurring algorithm, as it is presented in the pseudocode of the paper:
%
% D. Perrone, R. Diethelm and P. Favaro: "Blind Deconvolution via
% Lower-Bounded Logarithmic Image Priors", International Conference on
% Energy Minimization Methods in Computer Vision and Pattern Recognition
% (EMMCVPR), 2015.
%
% This script does not use the pyramide scheme but only a slow decreasing
% of the parameter lambda after each iteration.
% 
% Author: Daniele Perrone, perrone@iam.unibe.ch
% Copyright (C) 2014 Daniele Perrone. All rights reserved.

clear all
addpath('lib/');

ug = im2double(imread('peppers.png'));
ug = imresize(ug,[111 111]);
ug = mean(ug,3);

kg = fspecial('motion',7);
kg = padarray(kg,[3 0]);
MK = 7;
NK = 7;

f = convn(ug,kg,'valid');


params.tau = 1e-4;
params.k_step = 1e-3;
params.u_step = 1e-3;

params.outerIters = 1;
initialLambda = 5000;
finalLambda = 40000;

u = padarray(f,[floor(MK/2) floor(NK/2)],'replicate');
k = ones(MK,NK)/MK/NK;

lambda = initialLambda;
for i=1:10000
    params.u = u;
    params.k = k;
    [u,k] = blind(f, MK, NK, lambda, params);
    lambda = min(finalLambda,lambda * 1.001);
    
    if mod(i,50) == 0
        figure(1)
        imagesc([u ug]);
        drawnow
        
        figure(2)
        imagesc([k kg])
        drawnow
    end
end
